/*
 * acat.cpp:
 *
 * cat the contents of an AFF file...
 */

/*
 * Copyright (c) 2005
 *	Simson L. Garfinkel and Basis Technology, Inc. 
 *      All rights reserved.
 *
 * This code is derrived from software contributed by
 * Simson L. Garfinkel
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by Simson L. Garfinkel
 *    and Basis Technology Corp.
 * 4. Neither the name of Simson Garfinkel, Basis Technology, or other
 *    contributors to this program may be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY SIMSON GARFINKEL, BASIS TECHNOLOGY,
 * AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SIMSON GARFINKEL, BAIS TECHNOLOGy,
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.  
 */


#include "afflib.h"

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <err.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <string.h>
#include <zlib.h>
#include <openssl/md5.h>
#include <openssl/sha.h>
#include <assert.h>

char *progname = "acat";

int opt_info     = 0;

void usage()
{
    printf("usage: %s [options] infile [outfile]\n",progname);
    exit(0);
}



int main(int argc,char **argv)
{
    int bflag, ch;
     const char *infile;

     progname = argv[0];
     bflag = 0;
     while ((ch = getopt(argc, argv, "s:h?vi")) != -1) {
             switch (ch) {
	     case 'h':
             case '?':
             default:
		 usage();
	     }
     }
     argc -= optind;
     argv += optind;

     if(argc<1){
	 usage();
     }

     infile = *argv++;
     argc--;

     const char *outfilename = 0;
     if(argc>0){
	 outfilename = *argv++;
	 argc--;
     }

     FILE *outfile = stdout;

     if(outfilename){
	 outfile = fopen(outfilename,"wb");
	 if(!outfile) err(1,outfilename);
     }


     /* This needs to be restructured so that it reads segment by segment... */

     /* Just read from beginning to end of the file... */
     int total = 0;

     AFFILE *af = af_open(infile,O_RDONLY,0);
     if(!af) err(1,infile);
     size_t bytes;
     do {
	 char buf[65536];

	 bytes = af_read(af,(unsigned char *)buf,sizeof(buf));
	 if(bytes>0){
	     total += bytes;
	 }
	 /* tk: figure out which segment we were reading from... */
	 fwrite(buf,1,bytes,outfile);
     } while(bytes>0);

     /* Now --- here is the real question.
      * What should we do about segments that are in the image but were not read?
      */

     /* tk: read from the other segments... */

     af_close(af);
     fclose(outfile);
}
